@@ -4,6 +4,6 @@ function(object, ...)
   if(class(object)=="gets" || class(object)=="isat"){
     return(object$paths)
   }else{
-    cat("The object does not belong to the 'gets' nor 'isat' class\n")
+    stop("The object does not belong to the 'gets' nor 'isat' class")
   }
 }
